// History:
// 9-Oct-08 jcv SPRx2 enhanced delays.fit management
// 4-Feb-10 jcv SPRx22 gcc4 migration

#include "Delay.h"
#include "Utils.h"
#include "fdsorbit.h"

#include "CalOalUtility.h"



///////////////////////
//
//
//
//
Delay::Delay() {

  // SPR x2
  string sDelaysFilename;

  // Always present, I presume, as per stsp_settcx.sh
  if(std::getenv("TDLYFILENAME") == 0)
    errstr << "TDLYFILENAME is not defined " 
        <<  Sas::error("ERROR NoTDLYFILENAME");
  else
    sDelaysFilename = std::getenv("TDLYFILENAME");

  // Load delays into memory
  // SPR x2
  //DataSet *delayset = dataSetServer->open("delays.fit", Dal::Read);
  DataSet *delayset = dataSetServer->open(sDelaysFilename, Dal::Read);

  Table *xmcsTab = delayset->table("XMCS");

  Column *xstationCol = xmcsTab->column("STATION");
  Column *xtimeCol    = xmcsTab->column("TIME");
  Column *xdelayCol   = xmcsTab->column("DELAY");
 
  ColumnData *xstationColD = xstationCol->data();
  ColumnData *xtimeColD    = xtimeCol->data();
  ColumnData *xdelayColD   = xdelayCol->data();

  int16 *xstation = xstationColD->int16Data();
  real32 *xdelay  = xdelayColD->real32Data();

  for (unsigned int i = 0; i < xmcsTab->rows(); i++) {

    STime recordStart(CalOalUtility::stringColumnRow(xtimeColD, i));
    
    delay_pair entry(recordStart,xdelay[i]);

    xmcsDelays[xstation[i]].push_back(entry);
    
  }

  Table *actualTab = delayset->table("ACTUAL");

  Column *astationCol = actualTab->column("STATION");
  Column *atimeCol    = actualTab->column("TIME");
  Column *adelayCol   = actualTab->column("DELAY");
 
  ColumnData *astationColD = astationCol->data();
  ColumnData *atimeColD    = atimeCol->data();
  ColumnData *adelayColD   = adelayCol->data();

  int16 *astation = astationColD->int16Data();
  real32 *adelay  = adelayColD->real32Data();

  for (unsigned int i = 0; i < actualTab->rows(); i++) {

    STime recordStart(CalOalUtility::stringColumnRow(atimeColD, i));
    
    delay_pair entry(recordStart,adelay[i]);

    actualDelays[astation[i]].push_back(entry);

  }

  dataSetServer->close(delayset);
  
}



///////////////////////
//
//
//
//
double Delay::getSignalDelay(STime ertTime, unsigned char GSID) {
    
  // Input 
  double mjd = Utils::STime2MJD2000(ertTime);
  int orbit_lun = 68;
  int station_lun = 69;
  
  // Output
  int ierr;
  double delay = 0.0;
  int nsat;
 
  //SPRx22 gcc4 migration
  int iGSID=(int)GSID;
 
  sigdel_(orbit_lun,
	  station_lun,
          // SPRx22 gcc4 migration
	  //(int)GSID,
	  iGSID,
	  mjd,
	  &delay,
	  &nsat,
	  &ierr);
  
  if (ierr != 0) { 
    errstr << "Error from SIGDEL: " 
	   << ierr
	   << " GS = " 
	   << (int)GSID
	   << " MJD = "
	   << mjd
	   << Sas::warning("SIGDEL");  

    // Sometimes there is an interface problem with SIGDEL
    // such that the GSID is screwed. 
    // Retrying sometimes fixes the problem
    if (ierr == 4) {

      errstr << "Bad GSID in SIGDEL, retry" 
	     << Sas::message(Msg::App, Msg::Verbose);

      sigdel_(orbit_lun,
	      station_lun,
              // SPRx22 gcc4 migration
	      //(int)GSID,
	      iGSID,
	      mjd,
	      &delay,
	      &nsat,
	      &ierr);
      
      if (ierr != 0) { 
	errstr << "Again Error from SIGDEL: " 
	       << ierr
	       << " GS = " 
	       << (int)GSID
	       << " MJD = "
	       << mjd
	       << Sas::warning("SIGDEL2");  
	
      }
    }
  }

  return delay;
  
}



///////////////////////
//
//
//
//
double Delay::getXMCSGSdelay(STime ertTime, unsigned char GSID) {
  
  std::vector<delay_pair> station = xmcsDelays[GSID];
  
  double delay = 0.0;

  for (unsigned int i = 0; i < station.size(); i++) {

    if (ertTime > station[i].first) {

      delay = station[i].second;

    }

  }

  return delay;
  
}



///////////////////////
//
//
//
//
double Delay::getXMCSOBDHdelay(STime ertTime) {


  // GSID 0 is the CMDU
  std::vector<delay_pair> station = xmcsDelays[0];
  double delay = 0.0;
  
  for (unsigned int i = 0; i < station.size(); i++) {
    
    if (ertTime > station[i].first) {
      
      delay = station[i].second;
      
    }
    
  }
  
  return delay;
   
}



///////////////////////
//
//
//
//
double Delay::getActualGSdelay(STime ertTime, unsigned char GSID) {

  std::vector<delay_pair> station = actualDelays[GSID];
  
  double delay = 0.0;

  for (unsigned int i = 0; i < station.size(); i++) {

    if (ertTime > station[i].first) {

      delay = station[i].second;

    }

  }

  return delay;
  
}



///////////////////////
//
//
//
//
double Delay::getActualOBDHdelay(STime ertTime) {
  
  // GSID 0 is the CMDU
  std::vector<delay_pair> station = actualDelays[0];
  double delay = 0.0;
  
  for (unsigned int i = 0; i < station.size(); i++) {
    
    if (ertTime > station[i].first) {
      
      delay = station[i].second;
      
    }
    
  }
  
  return delay;
  
}
