// SPRx22 jcv 3-Feb-10 gcc4 migration


#include <iostream>

// SPRx22
#include <cmath>

#include "STime.h"

#include "Utils.h"

signed int Utils::sign(double d) {

  signed int i = 0;

  if (d == 0.0) {
    i = 0;
  }

  if (d < 0.0) {
    i = -1;
  }

  if (d > 0.0) {
    i = 1;
  }

  return i;

}

STime Utils::readVMSTime(Utils::vmsTime time) {

  Utils::vmsTime unixEpoch = time - Utils::UNIX_EPOCH0_IN_VMS_TIME;

  double seconds = (unixEpoch / 10000000) + (unixEpoch % 10000000) /  10000000.0;
  
  return STime("1970-01-01T00:00:00") + seconds;
  
}

STime Utils::readTDSTime(Utils::tdsTime time) {

  double seconds = (double)time.Seconds + ((double)time.Microseconds / 1.0e6);
  
  return STime("1970-01-01T00:00:00") + seconds;
  
}

STime Utils::cds2STime(CDSTime Time) {

  double dayFrac = 
    ((double)Time.Milliseconds * 1.0e-3 +
     (double)Time.Microseconds * 1.0e-6) / 86400.0;
  
  return STime(MJD2CDS + Time.Days + dayFrac);

}

Utils::CDSTime Utils::STime2cds(STime Time) {

  double day = Time.modifiedJulianDay().day();
  double fracday = Time.modifiedJulianDay().fractionalDay() * 86400.0 * 1.0e3;
  
  Utils::CDSTime cdsT;
  
  cdsT.Days = (int)day - MJD2CDS;
  cdsT.Milliseconds = (int)fracday;
  cdsT.Microseconds = (short)(rint((fracday - cdsT.Milliseconds) * 1.0e3));

  return cdsT;

}

double Utils::STime2MJD2000(STime time) {

  return time.modifiedJulianDay().full() - 51544.0;
  
}

STime Utils::MJD20002STime(double mjd2000) {
  
  return STime(mjd2000 + 51544.0);
  
}

void Utils::vmsByteSwap(unsigned char *buf, int size) {
  
#ifdef SOLARIS
  
  unsigned char tmp;
  
  if (size % 2 != 0) {
    std::cerr << "Uneven size buffer to swap. Not doing it" << std::endl;
    return;
  }
  
  for (int i = 0; i < size / 2; i++) {
    tmp = buf[i];
    buf[i] = buf[size - i - 1];
    buf[size - i - 1] = tmp;
  } 
  
#endif
  
#ifdef LINUX
  size = buf[0];
#endif
  
}
 
void Utils::sunByteSwap(unsigned char *buf, int size) {
  
#ifdef LINUX
  
  unsigned char tmp;
  
  if (size % 2 != 0) {
    std::cerr << "Uneven size buffer to swap. Not doing it" << std::endl;
    return;
  }

  for (int i = 0; i < size / 2; i++) {
    tmp = buf[i];
    buf[i] = buf[size - i - 1];
    buf[size - i - 1] = tmp;
  }
  
#endif
  
#ifdef SOLARIS
  size = buf[0];
#endif
  
}
